@@ -21,6 +21,7 @@
 import datetime
 import itertools
 import json
+import zoneinfo
 
 import croniter
 from oslo_config import cfg
@@ -30,10 +31,8 @@ from oslo_utils import timeutils
 from oslo_utils import uuidutils
 import pecan
 from pecan import rest
-import pytz
-import six
-from six.moves.urllib import parse as urlparse
 from stevedore import extension
+from urllib import parse as urlparse
 import wsme
 from wsme import types as wtypes
 import wsmeext.pecan as wsme_pecan
@@ -46,18 +45,20 @@ from aodh.i18n import _
 from aodh import keystone_client
 from aodh import messaging
 from aodh import notifier
+from aodh import profiler
 from aodh.storage import models
 
 LOG = log.getLogger(__name__)
 
-
 ALARM_API_OPTS = [
     cfg.IntOpt('user_alarm_quota',
                deprecated_group='DEFAULT',
+               default=-1,
                help='Maximum number of alarms defined for a user.'
                ),
     cfg.IntOpt('project_alarm_quota',
                deprecated_group='DEFAULT',
+               default=-1,
                help='Maximum number of alarms defined for a project.'
                ),
     cfg.IntOpt('alarm_max_actions',
@@ -75,6 +76,12 @@ severity_kind_enum = wtypes.Enum(str, *severity_kind)
 ALARM_REASON_DEFAULT = "Not evaluated yet"
 ALARM_REASON_MANUAL = "Manually set via API"
 
+ALARM_QUERY_FIELDS_ALLOWED = set([
+    'all_projects', 'user_id', 'project_id', 'type', 'name', 'enabled',
+    'state', 'severity', 'timestamp', 'repeat_actions'
+])
+ALARM_QUERY_OPS_ALLOWED = set(['eq'])
+
 
 class OverQuota(base.ClientSideError):
     def __init__(self, data):
@@ -97,25 +104,31 @@ def is_over_quota(conn, project_id, user_id):
 
     over_quota = False
 
-    # Start by checking for user quota
-    user_alarm_quota = pecan.request.cfg.api.user_alarm_quota
-    if user_alarm_quota is not None:
-        user_alarms = list(conn.get_alarms(user=user_id))
-        over_quota = len(user_alarms) >= user_alarm_quota
+    project_quotas = conn.get_quotas(project_id)
+    project_alarms = conn.get_alarms(project_id=project_id)
+    user_alarms = conn.get_alarms(user_id=user_id)
+    user_default_alarm_quota = pecan.request.cfg.api.user_alarm_quota
+    project_default_alarm_quota = pecan.request.cfg.api.project_alarm_quota
 
-    # If the user quota isn't reached, we check for the project quota
-    if not over_quota:
-        project_alarm_quota = pecan.request.cfg.api.project_alarm_quota
-        if project_alarm_quota is not None:
-            project_alarms = list(conn.get_alarms(project=project_id))
-            over_quota = len(project_alarms) >= project_alarm_quota
+    # 1. Check project quota
+    if len(project_quotas) > 0:
+        for quota in project_quotas:
+            if quota.resource == 'alarms':
+                over_quota = len(user_alarms) >= quota.limit
+    else:
+        if project_default_alarm_quota != -1:
+            over_quota = len(project_alarms) >= project_default_alarm_quota
+
+    # 2. Check user quota
+    if not over_quota and user_default_alarm_quota != -1:
+        over_quota = len(user_alarms) >= user_default_alarm_quota
 
     return over_quota
 
 
 class CronType(wtypes.UserType):
     """A user type that represents a cron format."""
-    basetype = six.string_types
+    basetype = str
     name = 'cron'
 
     @staticmethod
@@ -163,8 +176,8 @@ class AlarmTimeConstraint(base.Base):
     def validate(tc):
         if tc.timezone:
             try:
-                pytz.timezone(tc.timezone)
-            except Exception:
+                zoneinfo.ZoneInfo(tc.timezone)
+            except zoneinfo.ZoneInfoNotFoundError:
                 raise base.ClientSideError(_("Timezone %s is not valid")
                                            % tc.timezone)
         return tc
@@ -185,6 +198,7 @@ ACTIONS_SCHEMA = extension.ExtensionManager(
     notifier.AlarmNotifierService.NOTIFIER_EXTENSIONS_NAMESPACE).names()
 
 
+@profiler.trace_cls('api')
 class Alarm(base.Base):
     """Representation of an alarm."""
 
@@ -200,7 +214,7 @@ class Alarm(base.Base):
         rule = getattr(self, '%s_rule' % self.type, None)
         if not self._description:
             if hasattr(rule, 'default_description'):
-                return six.text_type(rule.default_description)
+                return str(rule.default_description)
             return "%s alarm rule" % self.type
         return self._description
 
@@ -258,6 +272,9 @@ class Alarm(base.Base):
                             default='low')
     "The severity of the alarm"
 
+    evaluate_timestamp = datetime.datetime
+    "The latest alarm evaluation time"
+
     def __init__(self, rule=None, time_constraints=None, **kwargs):
         super(Alarm, self).__init__(**kwargs)
 
@@ -339,7 +356,7 @@ class Alarm(base.Base):
                                             "maximum": max_actions}
                 raise base.ClientSideError(error)
 
-            limited = rbac.get_limited_to_project(pecan.request.headers,
+            limited = rbac.get_limited_to_project(pecan.request,
                                                   pecan.request.enforcer)
 
             for action in actions:
@@ -460,12 +477,14 @@ class Alarm(base.Base):
         if trust_id is not None and not trust_id_used:
             prev_trust_ids.add(trust_id)
         for old_trust_id in prev_trust_ids:
-            keystone_client.delete_trust_id(old_trust_id, auth_plugin)
+            keystone_client.delete_trust_id(pecan.request.cfg, old_trust_id,
+                                            auth_plugin)
 
     def delete_actions(self):
         auth_plugin = pecan.request.environ.get('keystone.token_auth')
         for trust_id in self._get_existing_trust_ids():
-            keystone_client.delete_trust_id(trust_id, auth_plugin)
+            keystone_client.delete_trust_id(pecan.request.cfg, trust_id,
+                                            auth_plugin)
 
 
 Alarm.add_attributes(**{"%s_rule" % ext.name: ext.plugin
@@ -529,9 +548,10 @@ def stringify_timestamps(data):
     """Stringify any datetimes in given dict."""
     return dict((k, v.isoformat()
                  if isinstance(v, datetime.datetime) else v)
-                for (k, v) in six.iteritems(data))
+                for (k, v) in data.items())
 
 
+@profiler.trace_cls('api')
 class AlarmController(rest.RestController):
     """Manages operations on a single alarm."""
 
@@ -545,18 +565,21 @@ class AlarmController(rest.RestController):
         self._id = alarm_id
 
     def _enforce_rbac(self, rbac_directive):
-        # TODO(sileht): We should be able to relax this since we
-        # pass the alarm object to the enforcer.
-        auth_project = rbac.get_limited_to_project(pecan.request.headers,
-                                                   pecan.request.enforcer)
-        alarms = list(pecan.request.storage.get_alarms(alarm_id=self._id,
-                                                       project=auth_project))
+        auth_project = pecan.request.headers.get('X-Project-Id')
+
+        filters = {'alarm_id': self._id}
+        is_admin = rbac.is_admin(pecan.request, pecan.request.enforcer)
+        if not is_admin:
+            filters['project_id'] = auth_project
+
+        alarms = pecan.request.storage.get_alarms(**filters)
         if not alarms:
-            raise base.AlarmNotFound(alarm=self._id, auth_project=auth_project)
+            raise base.AlarmNotFound(alarm=self._id, auth_project=None)
+
         alarm = alarms[0]
         target = {'user_id': alarm.user_id,
                   'project_id': alarm.project_id}
-        rbac.enforce(rbac_directive, pecan.request.headers,
+        rbac.enforce(rbac_directive, pecan.request,
                      pecan.request.enforcer, target)
         return alarm
 
@@ -630,7 +653,7 @@ class AlarmController(rest.RestController):
 
         data.alarm_id = self._id
 
-        user, project = rbac.get_limited_to(pecan.request.headers,
+        user, project = rbac.get_limited_to(pecan.request,
                                             pecan.request.enforcer)
         if user:
             data.user_id = user
@@ -695,7 +718,7 @@ class AlarmController(rest.RestController):
         # allow history to be returned for deleted alarms, but scope changes
         # returned to those carried out on behalf of the auth'd tenant, to
         # avoid inappropriate cross-tenant visibility of alarm history
-        auth_project = rbac.get_limited_to_project(pecan.request.headers,
+        auth_project = rbac.get_limited_to_project(pecan.request,
                                                    pecan.request.enforcer)
         conn = pecan.request.storage
         kwargs = v2_utils.query_to_kwargs(
@@ -738,6 +761,7 @@ class AlarmController(rest.RestController):
         return self._enforce_rbac('get_alarm_state').state
 
 
+@profiler.trace_cls('api')
 class AlarmsController(rest.RestController):
     """Manages operations on the alarms collection."""
 
@@ -780,14 +804,14 @@ class AlarmsController(rest.RestController):
 
         :param data: an alarm within the request body.
         """
-        rbac.enforce('create_alarm', pecan.request.headers,
+        rbac.enforce('create_alarm', pecan.request,
                      pecan.request.enforcer, {})
 
         conn = pecan.request.storage
         now = timeutils.utcnow()
 
         data.alarm_id = uuidutils.generate_uuid()
-        user_limit, project_limit = rbac.get_limited_to(pecan.request.headers,
+        user_limit, project_limit = rbac.get_limited_to(pecan.request,
                                                         pecan.request.enforcer)
 
         def _set_ownership(aspect, owner_limitation, header):
@@ -841,17 +865,56 @@ class AlarmsController(rest.RestController):
         :param marker: The pagination query marker.
         """
         target = rbac.target_from_segregation_rule(
-            pecan.request.headers, pecan.request.enforcer)
-        rbac.enforce('get_alarms', pecan.request.headers,
+            pecan.request, pecan.request.enforcer)
+        rbac.enforce('get_alarms', pecan.request,
                      pecan.request.enforcer, target)
 
         q = q or []
-        # Timestamp is not supported field for Simple Alarm queries
-        kwargs = v2_utils.query_to_kwargs(
-            q, pecan.request.storage.get_alarms,
-            allow_timestamps=False)
+        filters = {}
+
+        # Check field
+        keys = set([query.field for query in q])
+        if not keys.issubset(ALARM_QUERY_FIELDS_ALLOWED):
+            raise wsme.exc.InvalidInput(
+                'field', keys,
+                'only fields %s are allowed' % ALARM_QUERY_FIELDS_ALLOWED
+            )
+        # Check op
+        ops = set([query.op for query in q])
+        if any([op not in ALARM_QUERY_OPS_ALLOWED for op in ops]):
+            raise wsme.exc.InvalidInput(
+                'op', ops,
+                'only operations %s are allowed' % ALARM_QUERY_OPS_ALLOWED
+            )
+
+        if 'all_projects' in keys:
+            if v2_utils.get_query_value(q, 'all_projects', 'boolean'):
+                rbac.enforce('get_alarms:all_projects', pecan.request,
+                             pecan.request.enforcer, target)
+            keys.remove('all_projects')
+        else:
+            project_id = pecan.request.headers.get('X-Project-Id')
+            is_admin = rbac.is_admin(pecan.request, pecan.request.enforcer)
+
+            if not v2_utils.is_field_exist(q, 'project_id'):
+                q.append(
+                    base.Query(field='project_id', op='eq', value=project_id)
+                )
+                keys.add('project_id')
+            else:
+                request_project = v2_utils.get_query_value(q, 'project_id')
+                if not is_admin and request_project != project_id:
+                    raise base.ProjectNotAuthorized(request_project)
+
+        for query in q:
+            if query.field in keys:
+                filters[query.field] = {query.op: query.get_value(query.type)}
+
         if sort or limit or marker:
-            kwargs['pagination'] = v2_utils.get_pagination_options(
+            filters['pagination'] = v2_utils.get_pagination_options(
                 sort, limit, marker, models.Alarm)
+
+        LOG.debug('Getting alarms from database, filters: %s', filters)
+
         return [Alarm.from_db_model_scrubbed(m)
-                for m in pecan.request.storage.get_alarms(**kwargs)]
+                for m in pecan.request.storage.get_alarms(**filters)]
