@@ -15,12 +15,11 @@
 
 import copy
 import datetime
-import six
-import uuid
+import json
+from unittest import mock
 
-import mock
-from oslo_serialization import jsonutils
 from oslo_utils import timeutils
+from oslo_utils import uuidutils
 
 from aodh import evaluator
 from aodh.evaluator import event as event_evaluator
@@ -34,13 +33,14 @@ class TestEventAlarmEvaluate(base.TestEvaluatorBase):
 
     @staticmethod
     def _alarm(**kwargs):
-        alarm_id = kwargs.get('id') or str(uuid.uuid4())
+        alarm_id = kwargs.get('id') or uuidutils.generate_uuid()
         return models.Alarm(name=kwargs.get('name', alarm_id),
                             type='event',
                             enabled=True,
                             alarm_id=alarm_id,
                             description='desc',
                             state=kwargs.get('state', 'insufficient data'),
+                            state_reason='reason',
                             severity='critical',
                             state_timestamp=constants.MIN_DATETIME,
                             timestamp=constants.MIN_DATETIME,
@@ -56,7 +56,7 @@ class TestEventAlarmEvaluate(base.TestEvaluatorBase):
 
     @staticmethod
     def _event(**kwargs):
-        return {'message_id': kwargs.get('id') or str(uuid.uuid4()),
+        return {'message_id': kwargs.get('id') or uuidutils.generate_uuid(),
                 'event_type': kwargs.get('event_type', 'type0'),
                 'traits': kwargs.get('traits', [])}
 
@@ -65,7 +65,7 @@ class TestEventAlarmEvaluate(base.TestEvaluatorBase):
         self._update_history = []
 
         def get_alarms(**kwargs):
-            return (a for a in six.itervalues(self._stored_alarms))
+            return (a for a in self._stored_alarms.values())
 
         def update_alarm(alarm):
             self._stored_alarms[alarm.alarm_id] = copy.deepcopy(alarm)
@@ -99,13 +99,13 @@ class TestEventAlarmEvaluate(base.TestEvaluatorBase):
 
         if expect_db_queries is not None:
             expected = [mock.call(enabled=True,
-                                  alarm_type='event',
-                                  project=p) for p in expect_db_queries]
+                                  type='event',
+                                  project_id=p) for p in expect_db_queries]
             self.assertEqual(expected,
                              self.storage_conn.get_alarms.call_args_list)
 
         if expect_alarm_states is not None:
-            for alarm_id, state in six.iteritems(expect_alarm_states):
+            for alarm_id, state in expect_alarm_states.items():
                 self.assertEqual(state, self._stored_alarms[alarm_id].state)
 
         if expect_alarm_updates is not None:
@@ -127,8 +127,7 @@ class TestEventAlarmEvaluate(base.TestEvaluatorBase):
                           'query <query=%(query)s>.') % {
                     'e': event['message_id'],
                     'type': event['event_type'],
-                    'query': jsonutils.dumps(alarm.rule['query'],
-                                             sort_keys=True)}
+                    'query': json.dumps(alarm.rule['query'], sort_keys=True)}
                 data = {'type': 'event', 'event': event}
                 expected = dict(alarm_id=alarm.alarm_id,
                                 state=evaluator.ALARM,
@@ -193,7 +192,7 @@ class TestEventAlarmEvaluate(base.TestEvaluatorBase):
 
     def test_skip_event_missing_event_type(self):
         alarm = self._alarm()
-        event = {'message_id': str(uuid.uuid4()), 'traits': []}
+        event = {'message_id': uuidutils.generate_uuid(), 'traits': []}
         self._do_test_event_alarm(
             [alarm], [event],
             expect_alarm_states={alarm.alarm_id: evaluator.UNKNOWN},
